cdef extern from *:
    int PY_MAJOR_VERSION

cpdef bytes to_bytes(s)
cpdef unicode to_unicode(s)

cpdef inline to_string(s):
    r"""
    Converts a bytes and unicode ``s`` to a string.

    String means bytes in Python2 and unicode in Python3

    Examples:

    >>> from cypari2.string_utils import to_string
    >>> s1 = to_string(b'hello')
    >>> s2 = to_string('hello')
    >>> s3 = to_string(u'hello')
    >>> type(s1) == type(s2) == type(s3) == str
    True
    >>> s1 == s2 == s3 == 'hello'
    True
    """
    if PY_MAJOR_VERSION <= 2:
        return to_bytes(s)
    else:
        return to_unicode(s)



from .types cimport *
cimport cython

from .gen cimport Gen

# DEPRECATED INTERNAL FUNCTION used (incorrectly) in sagemath < 10.5
cpdef long prec_words_to_bits(long prec_in_words) noexcept
cpdef long default_bitprec() noexcept

cdef extern from *:
    """
    #define DEFAULT_BITPREC prec2nbits(DEFAULTPREC)
    """
    long DEFAULT_BITPREC

cdef class Pari_auto:
    pass

cdef class Pari(Pari_auto):
    cdef readonly Gen PARI_ZERO, PARI_ONE, PARI_TWO
    cpdef Gen zero(self)
    cpdef Gen one(self)
    cdef Gen _empty_vector(self, long n)

cdef long get_var(v) except -2



# This file is auto-generated by autogen/generator.py

from .types cimport *

cdef extern from *:
    GEN mpcatalan(long)
    GEN gtocol0(GEN, long)
    GEN gtocolrev0(GEN, long)
    GEN mpeuler(long)
    GEN gen_I()
    GEN gtolist(GEN)
    GEN gtomap(GEN)
    GEN gtomat(GEN)
    GEN gmodulo(GEN, GEN)
    GEN mppi(long)
    GEN gtopoly(GEN, long)
    GEN gtopolyrev(GEN, long)
    GEN Qfb0(GEN, GEN, GEN, GEN, long)
    GEN Ser0(GEN, long, GEN, long)
    GEN gtoset(GEN)
    GEN Strchr(GEN)
    GEN gtovec0(GEN, long)
    GEN gtovecrev0(GEN, long)
    GEN gtovecsmall0(GEN, long)
    GEN gabs(GEN, long)
    GEN gacos(GEN, long)
    GEN gacosh(GEN, long)
    void addhelp(char *, char *)
    GEN addprimes(GEN)
    GEN agm(GEN, GEN, long)
    GEN algadd(GEN, GEN, GEN)
    GEN algalgtobasis(GEN, GEN)
    GEN algaut(GEN)
    GEN algb(GEN)
    GEN algbasis(GEN)
    GEN algbasistoalg(GEN, GEN)
    GEN algcenter(GEN)
    GEN alg_centralproj(GEN, GEN, long)
    GEN algchar(GEN)
    GEN algcharpoly(GEN, GEN, long, long)
    long algdegree(GEN)
    GEN algdep0(GEN, long, long)
    long algdim(GEN, long)
    GEN algdisc(GEN)
    GEN algdivl(GEN, GEN, GEN)
    GEN algdivr(GEN, GEN, GEN)
    GEN alggroup(GEN, GEN)
    GEN alggroupcenter(GEN, GEN, GEN*)
    GEN alghasse(GEN, GEN)
    GEN alghassef(GEN)
    GEN alghassei(GEN)
    long algindex(GEN, GEN)
    GEN alginit(GEN, GEN, long, long)
    GEN alginv(GEN, GEN)
    GEN alginvbasis(GEN)
    int algisassociative(GEN, GEN)
    int algiscommutative(GEN)
    int algisdivision(GEN, GEN)
    int algisdivl(GEN, GEN, GEN, GEN*)
    int algisinv(GEN, GEN, GEN*)
    int algisramified(GEN, GEN)
    int algissemisimple(GEN)
    int algissimple(GEN, long)
    int algissplit(GEN, GEN)
    GEN alglatadd(GEN, GEN, GEN, GEN*)
    int alglatcontains(GEN, GEN, GEN, GEN*)
    GEN alglatelement(GEN, GEN, GEN)
    GEN alglathnf(GEN, GEN, GEN)
    GEN alglatindex(GEN, GEN, GEN)
    GEN alglatinter(GEN, GEN, GEN, GEN*)
    GEN alglatlefttransporter(GEN, GEN, GEN)
    GEN alglatmul(GEN, GEN, GEN)
    GEN alglatrighttransporter(GEN, GEN, GEN)
    int alglatsubset(GEN, GEN, GEN, GEN*)
    GEN algmakeintegral(GEN, long)
    GEN algmul(GEN, GEN, GEN)
    GEN algmultable(GEN)
    GEN algneg(GEN, GEN)
    GEN algnorm(GEN, GEN, long)
    GEN algpoleval(GEN, GEN, GEN)
    GEN algpow(GEN, GEN, GEN)
    GEN algprimesubalg(GEN)
    GEN alg_quotient(GEN, GEN, long)
    GEN algradical(GEN)
    GEN algramifiedplaces(GEN)
    GEN algrandom(GEN, GEN)
    GEN algrelmultable(GEN)
    GEN algsimpledec(GEN, long)
    GEN algsplit(GEN, long)
    GEN algsplittingdata(GEN)
    GEN algsplittingfield(GEN)
    GEN algsqr(GEN, GEN)
    GEN algsub(GEN, GEN, GEN)
    GEN algsubalg(GEN, GEN)
    GEN algtableinit(GEN, GEN)
    GEN algtensor(GEN, GEN, long)
    GEN algtomatrix(GEN, GEN, long)
    GEN algtrace(GEN, GEN, long)
    long algtype(GEN)
    GEN apply0(GEN, GEN)
    GEN garg(GEN, long)
    GEN gasin(GEN, long)
    GEN gasinh(GEN, long)
    GEN asympnum0(GEN, long, GEN, long)
    GEN gatan(GEN, long)
    GEN gatanh(GEN, long)
    GEN bernfrac(long)
    GEN bernpol(long, long)
    GEN bernreal(long, long)
    GEN bernvec(long)
    GEN hbessel1(GEN, GEN, long)
    GEN hbessel2(GEN, GEN, long)
    GEN ibessel(GEN, GEN, long)
    GEN jbessel(GEN, GEN, long)
    GEN jbesselh(GEN, GEN, long)
    GEN kbessel(GEN, GEN, long)
    GEN nbessel(GEN, GEN, long)
    GEN bestappr(GEN, GEN)
    GEN bestapprPade(GEN, long)
    GEN bestapprnf(GEN, GEN, GEN, long)
    GEN gcdext0(GEN, GEN)
    GEN polresultantext0(GEN, GEN, long)
    long bigomega(GEN)
    GEN binaire(GEN)
    GEN binomial0(GEN, GEN)
    GEN gbitand(GEN, GEN)
    GEN gbitneg(GEN, long)
    GEN gbitnegimply(GEN, GEN)
    GEN gbitor(GEN, GEN)
    GEN bitprecision0(GEN, long)
    GEN gbittest(GEN, long)
    GEN gbitxor(GEN, GEN)
    long bnfcertify0(GEN, long)
    GEN bnfcompress(GEN)
    GEN decodemodule(GEN, GEN)
    GEN bnfinit0(GEN, long, GEN, long)
    GEN bnfisintnorm(GEN, GEN)
    GEN bnfisnorm(GEN, GEN, long)
    GEN bnfisprincipal0(GEN, GEN, long)
    GEN bnfissunit(GEN, GEN, GEN)
    GEN bnfisunit(GEN, GEN)
    GEN bnflog(GEN, GEN)
    GEN bnflogdegree(GEN, GEN, GEN)
    GEN bnflogef(GEN, GEN)
    GEN bnfnarrow(GEN)
    GEN signunits(GEN)
    GEN bnfsunit(GEN, GEN, long)
    GEN bnrL1(GEN, GEN, long, long)
    GEN bnrchar(GEN, GEN, GEN)
    GEN bnrclassno0(GEN, GEN, GEN)
    GEN bnrclassnolist(GEN, GEN)
    GEN bnrconductor0(GEN, GEN, GEN, long)
    GEN bnrconductorofchar(GEN, GEN)
    GEN bnrdisc0(GEN, GEN, GEN, long)
    GEN bnrdisclist0(GEN, GEN, GEN)
    GEN bnrgaloisapply(GEN, GEN, GEN)
    GEN bnrgaloismatrix(GEN, GEN)
    GEN bnrinit0(GEN, GEN, long)
    long bnrisconductor0(GEN, GEN, GEN)
    long bnrisgalois(GEN, GEN, GEN)
    GEN bnrisprincipal(GEN, GEN, long)
    GEN bnrrootnumber(GEN, GEN, long, long)
    GEN bnrstark(GEN, GEN, long)
    GEN call0(GEN, GEN)
    GEN gceil(GEN)
    GEN centerlift0(GEN, long)
    GEN characteristic(GEN)
    GEN charconj0(GEN, GEN)
    GEN chardiv0(GEN, GEN, GEN)
    GEN chareval(GEN, GEN, GEN, GEN)
    GEN chargalois(GEN, GEN)
    GEN charker0(GEN, GEN)
    GEN charmul0(GEN, GEN, GEN)
    GEN charorder0(GEN, GEN)
    GEN charpoly0(GEN, long, long)
    GEN charpow0(GEN, GEN, GEN)
    GEN chinese(GEN, GEN)
    int cmp_universal(GEN, GEN)
    GEN compo(GEN, long)
    GEN gconcat(GEN, GEN)
    GEN gconj(GEN)
    GEN conjvec(GEN, long)
    GEN content0(GEN, GEN)
    GEN contfrac0(GEN, GEN, long)
    GEN contfraceval(GEN, GEN, long)
    GEN contfracinit(GEN, long)
    GEN contfracpnqn(GEN, long)
    GEN core0(GEN, long)
    GEN coredisc0(GEN, long)
    GEN gcos(GEN, long)
    GEN gcosh(GEN, long)
    GEN gcotan(GEN, long)
    GEN gcotanh(GEN, long)
    GEN default0(char *, char *)
    GEN denominator(GEN, GEN)
    GEN deriv(GEN, long)
    GEN diffop0(GEN, GEN, GEN, long)
    GEN digits(GEN, GEN)
    GEN dilog(GEN, long)
    GEN dirdiv(GEN, GEN)
    GEN dirmul(GEN, GEN)
    GEN dirzetak(GEN, GEN)
    GEN divisors0(GEN, long)
    GEN divisorslenstra(GEN, GEN, GEN)
    GEN divrem(GEN, GEN, long)
    GEN veceint1(GEN, GEN, long)
    GEN ellL1_bitprec(GEN, long, long)
    GEN elladd(GEN, GEN, GEN)
    GEN akell(GEN, GEN)
    GEN ellan(GEN, long)
    GEN ellanalyticrank_bitprec(GEN, GEN, long)
    GEN ellap(GEN, GEN)
    GEN bilhell(GEN, GEN, GEN, long)
    GEN ellbsd(GEN, long)
    GEN ellcard(GEN, GEN)
    GEN ellchangecurve(GEN, GEN)
    GEN ellchangepoint(GEN, GEN)
    GEN ellchangepointinv(GEN, GEN)
    GEN ellconvertname(GEN)
    GEN elldivpol(GEN, long, long)
    GEN elleisnum(GEN, long, long, long)
    GEN elleta(GEN, long)
    GEN ellformaldifferential(GEN, long, long)
    GEN ellformalexp(GEN, long, long)
    GEN ellformallog(GEN, long, long)
    GEN ellformalpoint(GEN, long, long)
    GEN ellformalw(GEN, long, long)
    GEN ellfromeqn(GEN)
    GEN ellfromj(GEN)
    GEN ellgenerators(GEN)
    GEN ellglobalred(GEN)
    GEN ellgroup0(GEN, GEN, long)
    GEN ellheegner(GEN)
    GEN ellheight0(GEN, GEN, GEN, long)
    GEN ellheightmatrix(GEN, GEN, long)
    GEN ellidentify(GEN)
    GEN ellinit(GEN, GEN, long)
    GEN ellintegralmodel(GEN, GEN*)
    long ellisdivisible(GEN, GEN, GEN, GEN*)
    GEN ellisogeny(GEN, GEN, long, long, long)
    GEN ellisogenyapply(GEN, GEN)
    GEN ellisomat(GEN, long, long)
    GEN ellisoncurve(GEN, GEN)
    GEN ellisotree(GEN)
    int ellissupersingular(GEN, GEN)
    GEN jell(GEN, long)
    GEN elllocalred(GEN, GEN)
    GEN elllog(GEN, GEN, GEN, GEN)
    GEN elllseries(GEN, GEN, GEN, long)
    GEN ellminimaldisc(GEN)
    GEN ellminimalmodel(GEN, GEN*)
    GEN ellminimaltwist0(GEN, long)
    GEN ellmoddegree(GEN)
    GEN ellmodulareqn(long, long, long)
    GEN ellmul(GEN, GEN, GEN)
    GEN ellneg(GEN, GEN)
    GEN ellnonsingularmultiple(GEN, GEN)
    GEN ellorder(GEN, GEN, GEN)
    GEN ellordinate(GEN, GEN, long)
    GEN ellpadicL(GEN, GEN, long, GEN, long, GEN)
    GEN ellpadicbsd(GEN, GEN, long, GEN)
    GEN ellpadicfrobenius(GEN, long, long)
    GEN ellpadicheight0(GEN, GEN, long, GEN, GEN)
    GEN ellpadicheightmatrix(GEN, GEN, long, GEN)
    GEN ellpadiclog(GEN, GEN, long, GEN)
    GEN ellpadicregulator(GEN, GEN, long, GEN)
    GEN ellpadics2(GEN, GEN, long)
    GEN ellperiods(GEN, long, long)
    GEN zell(GEN, GEN, long)
    GEN ellmul(GEN, GEN, GEN)
    GEN ellratpoints(GEN, GEN, long)
    long ellrootno(GEN, GEN)
    GEN ellsea(GEN, long)
    GEN ellsearch(GEN)
    GEN ellsigma(GEN, GEN, long, long)
    GEN ellsub(GEN, GEN, GEN)
    GEN elltamagawa(GEN)
    GEN elltaniyama(GEN, long)
    GEN elltatepairing(GEN, GEN, GEN, GEN)
    GEN elltors(GEN)
    GEN elltwist(GEN, GEN)
    GEN ellweilcurve(GEN, GEN*)
    GEN ellweilpairing(GEN, GEN, GEN, GEN)
    GEN ellwp0(GEN, GEN, long, long)
    GEN ellxn(GEN, long, long)
    GEN ellzeta(GEN, GEN, long)
    GEN pointell(GEN, GEN, long)
    GEN gerfc(GEN, long)
    GEN errname(GEN)
    GEN eta0(GEN, long, long)
    GEN eulerphi(GEN)
    GEN gexp(GEN, long)
    GEN gexpm1(GEN, long)
    GEN gpexponent(GEN)
    GEN gpextern(char *)
    GEN externstr(char *)
    GEN factor0(GEN, GEN)
    GEN factorback2(GEN, GEN)
    GEN factmod(GEN, GEN)
    GEN factorff(GEN, GEN, GEN)
    GEN mpfactr(long, long)
    GEN factorint(GEN, long)
    GEN factormod0(GEN, GEN, long)
    GEN factormodDDF(GEN, GEN)
    GEN factormodSQF(GEN, GEN)
    GEN polfnf(GEN, GEN)
    GEN factorpadic(GEN, GEN, long)
    GEN ffcompomap(GEN, GEN)
    GEN ffembed(GEN, GEN)
    GEN ffextend(GEN, GEN, long)
    GEN fffrobenius(GEN, long)
    GEN ffgen(GEN, long)
    GEN ffinit(GEN, long, long)
    GEN ffinvmap(GEN)
    GEN fflog(GEN, GEN, GEN)
    GEN ffmap(GEN, GEN)
    GEN ffnbirred0(GEN, long, long)
    GEN fforder(GEN, GEN)
    GEN ffprimroot(GEN, GEN*)
    GEN fibo(long)
    void gp_fileclose(long)
    long gp_fileextern(char *)
    void gp_fileflush0(GEN)
    long gp_fileopen(char *, char *)
    GEN gp_fileread(long)
    GEN gp_filereadstr(long)
    void gp_filewrite(long, char *)
    void gp_filewrite1(long, char *)
    GEN gfloor(GEN)
    GEN fold0(GEN, GEN)
    GEN gfrac(GEN)
    GEN fromdigits(GEN, GEN)
    GEN galoischardet(GEN, GEN, long)
    GEN galoischarpoly(GEN, GEN, long)
    GEN galoischartable(GEN)
    GEN galoisconjclasses(GEN)
    GEN galoisexport(GEN, long)
    GEN galoisfixedfield(GEN, GEN, long, long)
    GEN galoisgetgroup(long, long)
    GEN galoisgetname(long, long)
    GEN galoisgetpol(long, long, long)
    GEN galoisidentify(GEN)
    GEN galoisinit(GEN, GEN)
    GEN galoisisabelian(GEN, long)
    long galoisisnormal(GEN, GEN)
    GEN galoispermtopol(GEN, GEN)
    GEN galoissubcyclo(GEN, GEN, long, long)
    GEN galoissubfields(GEN, long, long)
    GEN galoissubgroups(GEN)
    GEN ggamma(GEN, long)
    GEN ggammah(GEN, long)
    GEN gammamellininv(GEN, GEN, long, long)
    GEN gammamellininvasymp(GEN, long, long)
    GEN gammamellininvinit(GEN, long, long)
    GEN ggcd0(GEN, GEN)
    GEN gcdext0(GEN, GEN)
    GEN genus2red(GEN, GEN)
    long getabstime()
    GEN getcache()
    GEN gp_getenv(char *)
    GEN getheap()
    GEN getrand()
    long getstack()
    long gettime()
    GEN getwalltime()
    long hammingweight(GEN)
    long hilbert(GEN, GEN, GEN)
    GEN hyperellcharpoly(GEN)
    GEN hyperellpadicfrobenius(GEN, unsigned long, long)
    GEN hyperellratpoints(GEN, GEN, long)
    GEN hyperu(GEN, GEN, GEN, long)
    GEN idealadd(GEN, GEN, GEN)
    GEN idealaddtoone0(GEN, GEN, GEN)
    GEN idealappr0(GEN, GEN, long)
    GEN idealchinese(GEN, GEN, GEN)
    GEN idealcoprime(GEN, GEN, GEN)
    GEN idealdiv0(GEN, GEN, GEN, long)
    GEN gpidealfactor(GEN, GEN, GEN)
    GEN idealfactorback(GEN, GEN, GEN, long)
    GEN idealfrobenius(GEN, GEN, GEN)
    GEN idealhnf0(GEN, GEN, GEN)
    GEN idealintersect(GEN, GEN, GEN)
    GEN idealinv(GEN, GEN)
    long idealispower(GEN, GEN, long, GEN*)
    GEN ideallist0(GEN, long, long)
    GEN ideallistarch(GEN, GEN, GEN)
    GEN ideallog(GEN, GEN, GEN)
    GEN idealmin(GEN, GEN, GEN)
    GEN idealmul0(GEN, GEN, GEN, long)
    GEN idealnorm(GEN, GEN)
    GEN idealnumden(GEN, GEN)
    GEN idealpow0(GEN, GEN, GEN, long)
    GEN idealprimedec_limit_f(GEN, GEN, long)
    GEN idealprincipalunits(GEN, GEN, long)
    GEN idealramgroups(GEN, GEN, GEN)
    GEN idealred0(GEN, GEN, GEN)
    GEN idealredmodpower(GEN, GEN, unsigned long, unsigned long)
    GEN idealstar0(GEN, GEN, long)
    GEN idealtwoelt0(GEN, GEN, GEN)
    GEN gpidealval(GEN, GEN, GEN)
    GEN gimag(GEN)
    GEN incgam0(GEN, GEN, GEN, long)
    GEN incgamc(GEN, GEN, long)
    GEN gp_input()
    void gpinstall(char *, char *, char *, char *)
    GEN integ(GEN, long)
    GEN intnumgaussinit(long, long)
    GEN intnuminit(GEN, GEN, long, long)
    long isfundamental(GEN)
    long ispolygonal(GEN, GEN, GEN*)
    long ispower(GEN, GEN, GEN*)
    long ispowerful(GEN)
    GEN gisprime(GEN, long)
    long isprimepower(GEN, GEN*)
    GEN gispseudoprime(GEN, long)
    long ispseudoprimepower(GEN, GEN*)
    long issquareall(GEN, GEN*)
    long issquarefree(GEN)
    long istotient(GEN, GEN*)
    void kill0(char *)
    long kronecker(GEN, GEN)
    GEN glambertW(GEN, long)
    GEN laurentseries0(GEN, long, long, long)
    GEN glcm0(GEN, GEN)
    long glength(GEN)
    int lexcmp(GEN, GEN)
    GEN lfun0(GEN, GEN, long, long)
    GEN lfunabelianrelinit(GEN, GEN, GEN, GEN, long, long)
    GEN lfunan(GEN, long, long)
    GEN lfunartin(GEN, GEN, GEN, long, long)
    long lfuncheckfeq(GEN, GEN, long)
    GEN lfunconductor(GEN, GEN, long, long)
    GEN lfuncost0(GEN, GEN, long, long)
    GEN lfuncreate(GEN)
    GEN lfundiv(GEN, GEN, long)
    GEN lfunetaquo(GEN)
    GEN lfungenus2(GEN)
    GEN lfunhardy(GEN, GEN, long)
    GEN lfuninit0(GEN, GEN, long, long)
    GEN lfunlambda0(GEN, GEN, long, long)
    GEN lfunmf(GEN, GEN, long)
    GEN lfunmfspec(GEN, long)
    GEN lfunmul(GEN, GEN, long)
    long lfunorderzero(GEN, long, long)
    GEN lfunqf(GEN, long)
    GEN lfunrootres(GEN, long)
    GEN lfunsympow(GEN, unsigned long)
    GEN lfuntheta(GEN, GEN, long, long)
    long lfunthetacost0(GEN, GEN, long, long)
    GEN lfunthetainit(GEN, GEN, long, long)
    GEN lfuntwist(GEN, GEN)
    GEN lfunzeros(GEN, GEN, long, long)
    GEN lift0(GEN, long)
    GEN liftall(GEN)
    GEN liftint(GEN)
    GEN liftpol(GEN)
    GEN limitnum0(GEN, long, GEN, long)
    GEN lindep0(GEN, long)
    GEN listinsert(GEN, GEN, long)
    void listkill(GEN)
    void listpop0(GEN, long)
    GEN listput0(GEN, GEN, long)
    void listsort(GEN, long)
    GEN glngamma(GEN, long)
    void localbitprec(long)
    void localprec(long)
    GEN glog(GEN, long)
    GEN glog1p(GEN, long)
    long logint0(GEN, GEN, GEN*)
    void mapdelete(GEN, GEN)
    GEN mapget(GEN, GEN)
    int mapisdefined(GEN, GEN, GEN*)
    void mapput(GEN, GEN, GEN)
    GEN matadjoint0(GEN, long)
    GEN matalgtobasis(GEN, GEN)
    GEN matbasistoalg(GEN, GEN)
    GEN matcompanion(GEN)
    GEN matconcat(GEN)
    GEN det0(GEN, long)
    GEN detint(GEN)
    GEN matdetmod(GEN, GEN)
    GEN diagonal(GEN)
    GEN mateigen(GEN, long, long)
    GEN matfrobenius(GEN, long, long)
    GEN hess(GEN)
    GEN mathilbert(long)
    GEN mathnf0(GEN, long)
    GEN hnfmod(GEN, GEN)
    GEN hnfmodid(GEN, GEN)
    GEN mathouseholder(GEN, GEN)
    GEN matid(long)
    GEN matimage0(GEN, long)
    GEN imagecompl(GEN)
    GEN matimagemod(GEN, GEN, GEN*)
    GEN indexrank(GEN)
    GEN intersect(GEN, GEN)
    GEN inverseimage(GEN, GEN)
    GEN matinvmod(GEN, GEN)
    int isdiagonal(GEN)
    GEN matker0(GEN, long)
    GEN matkerint0(GEN, long)
    GEN matkermod(GEN, GEN, GEN*)
    GEN matmuldiagonal(GEN, GEN)
    GEN matmultodiagonal(GEN, GEN)
    GEN matqpascal(long, GEN)
    GEN matpermanent(GEN)
    GEN matqr(GEN, long, long)
    long rank(GEN)
    GEN matrixqz0(GEN, GEN)
    GEN matsize(GEN)
    GEN matsnf0(GEN, long)
    GEN gauss(GEN, GEN)
    GEN matsolvemod(GEN, GEN, GEN, long)
    GEN suppl(GEN)
    GEN gtrans(GEN)
    GEN gmax(GEN, GEN)
    GEN mfDelta()
    GEN mfEH(GEN)
    GEN mfEk(long)
    GEN mfTheta(GEN)
    GEN mfatkin(GEN, GEN)
    GEN mfatkineigenvalues(GEN, long, long)
    GEN mfatkininit(GEN, long, long)
    GEN mfbasis(GEN, long)
    GEN mfbd(GEN, long)
    GEN mfbracket(GEN, GEN, long)
    GEN mfcoef(GEN, long)
    GEN mfcoefs(GEN, long, long)
    long mfconductor(GEN, GEN)
    GEN mfcosets(GEN)
    long mfcuspisregular(GEN, GEN)
    GEN mfcusps(GEN)
    GEN mfcuspval(GEN, GEN, GEN, long)
    long mfcuspwidth(GEN, GEN)
    GEN mfderiv(GEN, long)
    GEN mfderivE2(GEN, long)
    GEN mfdescribe(GEN, GEN*)
    GEN mfdim(GEN, long)
    GEN mfdiv(GEN, GEN)
    GEN mfeigenbasis(GEN)
    GEN mfeigensearch(GEN, GEN)
    GEN mfeisenstein(long, GEN, GEN)
    GEN mfembed0(GEN, GEN, long)
    GEN mfeval(GEN, GEN, GEN, long)
    GEN mffields(GEN)
    GEN mffromell(GEN)
    GEN mffrometaquo(GEN, long)
    GEN mffromlfun(GEN, long)
    GEN mffromqf(GEN, GEN)
    GEN mfgaloistype(GEN, GEN)
    GEN mfhecke(GEN, GEN, long)
    GEN mfheckemat(GEN, GEN)
    GEN mfinit(GEN, long)
    GEN mfisCM(GEN)
    long mfisequal(GEN, GEN, long)
    GEN mfkohnenbasis(GEN)
    GEN mfkohnenbijection(GEN)
    GEN mfkohneneigenbasis(GEN, GEN)
    GEN mflinear(GEN, GEN)
    GEN mfmanin(GEN, long)
    GEN mfmul(GEN, GEN)
    GEN mfnumcusps(GEN)
    GEN mfparams(GEN)
    GEN mfperiodpol(GEN, GEN, long, long)
    GEN mfperiodpolbasis(long, long)
    GEN mfpetersson(GEN, GEN)
    GEN mfpow(GEN, long)
    GEN mfsearch(GEN, GEN, long)
    GEN mfshift(GEN, long)
    GEN mfshimura(GEN, GEN, long)
    GEN mfslashexpansion(GEN, GEN, GEN, long, long, GEN*, long)
    long mfspace(GEN, GEN)
    GEN mfsplit(GEN, long, long)
    long mfsturm(GEN)
    GEN mfsymbol(GEN, GEN, long)
    GEN mfsymboleval(GEN, GEN, GEN, long)
    GEN mftaylor(GEN, long, long, long)
    GEN mftobasis(GEN, GEN, long)
    GEN mftocoset(long, GEN, GEN)
    GEN mftonew(GEN, GEN)
    GEN mftraceform(GEN, long)
    GEN mftwist(GEN, GEN)
    GEN gmin(GEN, GEN)
    GEN minpoly(GEN, long)
    GEN modreverse(GEN)
    long moebius(GEN)
    GEN msatkinlehner(GEN, long, GEN)
    GEN mscuspidal(GEN, long)
    long msdim(GEN)
    GEN mseisenstein(GEN)
    GEN mseval(GEN, GEN, GEN)
    GEN msfromcusp(GEN, GEN)
    GEN msfromell(GEN, long)
    GEN msfromhecke(GEN, GEN, GEN)
    long msgetlevel(GEN)
    long msgetsign(GEN)
    long msgetweight(GEN)
    GEN mshecke(GEN, long, GEN)
    GEN msinit(GEN, GEN, long)
    GEN msissymbol(GEN, GEN)
    GEN mslattice(GEN, GEN)
    GEN msnew(GEN)
    GEN msomseval(GEN, GEN, GEN)
    GEN mspadicL(GEN, GEN, long)
    GEN mspadicinit(GEN, long, long, long)
    GEN mspadicmoments(GEN, GEN, long)
    GEN mspadicseries(GEN, long)
    GEN mspathgens(GEN)
    GEN mspathlog(GEN, GEN)
    GEN mspetersson(GEN, GEN, GEN)
    GEN mspolygon(GEN, long)
    GEN msqexpansion(GEN, GEN, long)
    GEN mssplit(GEN, GEN, long)
    GEN msstar(GEN, GEN)
    GEN mstooms(GEN, GEN)
    GEN newtonpoly(GEN, GEN)
    GEN nextprime(GEN)
    GEN algtobasis(GEN, GEN)
    GEN nfbasis_gp(GEN)
    GEN basistoalg(GEN, GEN)
    GEN nfcertify(GEN)
    GEN nfcompositum(GEN, GEN, GEN, long)
    GEN nfdetint(GEN, GEN)
    GEN nfdisc(GEN)
    GEN nfadd(GEN, GEN, GEN)
    GEN nfdiv(GEN, GEN, GEN)
    GEN nfdiveuc(GEN, GEN, GEN)
    GEN nfdivmodpr(GEN, GEN, GEN, GEN)
    GEN nfdivrem(GEN, GEN, GEN)
    GEN nfeltembed(GEN, GEN, GEN, long)
    GEN nfmod(GEN, GEN, GEN)
    GEN nfmul(GEN, GEN, GEN)
    GEN nfmulmodpr(GEN, GEN, GEN, GEN)
    GEN nfnorm(GEN, GEN)
    GEN nfpow(GEN, GEN, GEN)
    GEN nfpowmodpr(GEN, GEN, GEN, GEN)
    GEN nfreduce(GEN, GEN, GEN)
    GEN nfreducemodpr(GEN, GEN, GEN)
    GEN nfeltsign(GEN, GEN, GEN)
    GEN nftrace(GEN, GEN)
    GEN gpnfvalrem(GEN, GEN, GEN, GEN*)
    GEN nffactor(GEN, GEN)
    GEN nffactorback(GEN, GEN, GEN)
    GEN nffactormod(GEN, GEN, GEN)
    GEN galoisapply(GEN, GEN, GEN)
    GEN galoisconj0(GEN, long, GEN, long)
    GEN nfgrunwaldwang(GEN, GEN, GEN, GEN, long)
    long nfhilbert0(GEN, GEN, GEN, GEN)
    GEN nfhnf0(GEN, GEN, long)
    GEN nfhnfmod(GEN, GEN, GEN)
    GEN nfinit0(GEN, long, long)
    long isideal(GEN, GEN)
    GEN nfisincl(GEN, GEN)
    GEN nfisisom(GEN, GEN)
    long nfislocalpower(GEN, GEN, GEN, GEN)
    GEN nfkermodpr(GEN, GEN, GEN)
    GEN nfmodpr(GEN, GEN, GEN)
    GEN nfmodprinit(GEN, GEN)
    GEN nfmodprlift(GEN, GEN, GEN)
    GEN nfnewprec(GEN, long)
    GEN nfpolsturm(GEN, GEN, GEN)
    GEN nfroots(GEN, GEN)
    GEN rootsof1(GEN)
    GEN nfsnf0(GEN, GEN, long)
    GEN nfsolvemodpr(GEN, GEN, GEN, GEN)
    GEN nfsplitting(GEN, GEN)
    GEN nfsubfields(GEN, long)
    GEN gnorm(GEN)
    GEN gnorml2(GEN)
    GEN gnormlp(GEN, GEN, long)
    GEN numbpart(GEN)
    GEN numdiv(GEN)
    GEN numerator(GEN, GEN)
    GEN numtoperm(long, GEN)
    long omega(GEN)
    GEN mkoo()
    GEN padicappr(GEN, GEN)
    GEN padicfields0(GEN, GEN, long)
    GEN gppadicprec(GEN, GEN)
    GEN parapply(GEN, GEN)
    GEN pareval(GEN)
    GEN parselect(GEN, GEN, long)
    GEN partitions(long, GEN, GEN)
    long permorder(GEN)
    long permsign(GEN)
    GEN permtonum(GEN)
    void plotbox(long, GEN, GEN, long)
    void plotclip(long)
    GEN plotcolor(long, GEN)
    void plotcopy(long, long, GEN, GEN, long)
    GEN plotcursor(long)
    void plotdraw(GEN, long)
    GEN plotexport(GEN, GEN, long)
    GEN plothraw(GEN, GEN, long)
    GEN plothrawexport(GEN, GEN, GEN, long)
    GEN plothsizes(long)
    void plotinit(long, GEN, GEN, long)
    void plotkill(long)
    void plotlines(long, GEN, GEN, long)
    void plotlinetype(long, long)
    void plotmove(long, GEN, GEN)
    void plotpoints(long, GEN, GEN)
    void plotpointsize(long, GEN)
    void plotpointtype(long, long)
    void plotrbox(long, GEN, GEN, long)
    GEN plotrecthraw(long, GEN, long)
    void plotrline(long, GEN, GEN)
    void plotrmove(long, GEN, GEN)
    void plotrpoint(long, GEN, GEN)
    void plotscale(long, GEN, GEN, GEN, GEN)
    void plotstring(long, char *, long)
    GEN polchebyshev_eval(long, long, GEN)
    GEN polclass(GEN, long, long)
    GEN polcoef(GEN, long, long)
    GEN polcoeff0(GEN, long, long)
    GEN polcompositum0(GEN, GEN, long)
    GEN polcyclo_eval(long, GEN)
    GEN polcyclofactors(GEN)
    GEN gppoldegree(GEN, long)
    GEN poldisc0(GEN, long)
    GEN poldiscfactors(GEN, long)
    GEN reduceddiscsmith(GEN)
    GEN polgalois(GEN, long)
    GEN polgraeffe(GEN)
    GEN polhensellift(GEN, GEN, GEN, long)
    GEN polhermite_eval(long, GEN)
    GEN polint(GEN, GEN, GEN, GEN*)
    long poliscyclo(GEN)
    long poliscycloprod(GEN)
    long isirreducible(GEN)
    GEN pollead(GEN, long)
    GEN pollegendre_eval(long, GEN)
    GEN polmodular(long, long, GEN, long, long)
    GEN polrecip(GEN)
    GEN polred0(GEN, long, GEN)
    GEN polredabs0(GEN, long)
    GEN polredbest(GEN, long)
    GEN polredord(GEN)
    GEN polresultant0(GEN, GEN, long, long)
    GEN polresultantext0(GEN, GEN, long)
    GEN roots(GEN, long)
    GEN polrootsbound(GEN, GEN)
    GEN polrootsff(GEN, GEN, GEN)
    GEN polrootsmod(GEN, GEN)
    GEN rootpadic(GEN, GEN, long)
    GEN realroots(GEN, GEN, long)
    long sturmpart(GEN, GEN, GEN)
    GEN polsubcyclo(long, long, long)
    GEN sylvestermatrix(GEN, GEN)
    GEN polsym(GEN, long)
    GEN polchebyshev1(long, long)
    GEN tschirnhaus(GEN)
    GEN polylog0(long, GEN, long, long)
    GEN polzag(long, long)
    GEN gpowers0(GEN, long, GEN)
    GEN precision0(GEN, long)
    GEN precprime(GEN)
    GEN prime(long)
    GEN primecert(GEN, long)
    GEN primecertexport(GEN, long)
    long primecertisvalid(GEN)
    GEN primepi(GEN)
    GEN primes0(GEN)
    GEN prodeulerrat(GEN, GEN, long, long)
    GEN prodnumrat(GEN, long, long)
    void psdraw(GEN, long)
    GEN gpsi(GEN, long)
    GEN psplothraw(GEN, GEN, long)
    GEN qfauto0(GEN, GEN)
    GEN qfautoexport(GEN, long)
    GEN qfbclassno0(GEN, long)
    GEN qfbcompraw(GEN, GEN)
    GEN hclassno(GEN)
    GEN qfbil(GEN, GEN, GEN)
    GEN nucomp(GEN, GEN, GEN)
    GEN nupow(GEN, GEN, GEN)
    GEN qfbpowraw(GEN, long)
    GEN primeform(GEN, GEN, long)
    GEN qfbred0(GEN, long, GEN, GEN, GEN)
    GEN qfbredsl2(GEN, GEN)
    GEN qfbsolve(GEN, GEN)
    GEN qfeval0(GEN, GEN, GEN)
    GEN qfgaussred(GEN)
    GEN qfisom0(GEN, GEN, GEN, GEN)
    GEN qfisominit0(GEN, GEN, GEN)
    GEN jacobi(GEN, long)
    GEN qflll0(GEN, long)
    GEN qflllgram0(GEN, long)
    GEN qfminim0(GEN, GEN, GEN, long, long)
    GEN qfnorm(GEN, GEN)
    GEN qforbits(GEN, GEN)
    GEN qfparam(GEN, GEN, long)
    GEN perf(GEN)
    GEN qfrep0(GEN, GEN, long)
    GEN qfsign(GEN)
    GEN qfsolve(GEN)
    GEN quadclassunit0(GEN, long, GEN, long)
    GEN quaddisc(GEN)
    GEN quadgen0(GEN, long)
    GEN quadhilbert(GEN, long)
    GEN quadpoly0(GEN, long)
    GEN quadray(GEN, GEN, long)
    GEN quadregulator(GEN, long)
    GEN quadunit0(GEN, long)
    GEN ramanujantau(GEN)
    GEN genrand(GEN)
    GEN randomprime(GEN)
    GEN gp_read_file(char *)
    GEN readstr(char *)
    GEN gp_readvec_file(char *)
    GEN greal(GEN)
    GEN removeprimes(GEN)
    GEN rnfalgtobasis(GEN, GEN)
    GEN rnfbasis(GEN, GEN)
    GEN rnfbasistoalg(GEN, GEN)
    GEN rnfcharpoly(GEN, GEN, GEN, long)
    GEN rnfconductor(GEN, GEN)
    GEN rnfdedekind(GEN, GEN, GEN, long)
    GEN rnfdet(GEN, GEN)
    GEN rnfdiscf(GEN, GEN)
    GEN rnfeltabstorel(GEN, GEN)
    GEN rnfeltdown0(GEN, GEN, long)
    GEN rnfeltnorm(GEN, GEN)
    GEN rnfeltreltoabs(GEN, GEN)
    GEN rnfelttrace(GEN, GEN)
    GEN rnfeltup0(GEN, GEN, long)
    GEN rnfequation0(GEN, GEN, long)
    GEN rnfhnfbasis(GEN, GEN)
    GEN rnfidealabstorel(GEN, GEN)
    GEN rnfidealdown(GEN, GEN)
    GEN rnfidealfactor(GEN, GEN)
    GEN rnfidealhnf(GEN, GEN)
    GEN rnfidealmul(GEN, GEN, GEN)
    GEN rnfidealnormabs(GEN, GEN)
    GEN rnfidealnormrel(GEN, GEN)
    GEN rnfidealprimedec(GEN, GEN)
    GEN rnfidealreltoabs0(GEN, GEN, long)
    GEN rnfidealtwoelement(GEN, GEN)
    GEN rnfidealup0(GEN, GEN, long)
    GEN rnfinit0(GEN, GEN, long)
    long rnfisabelian(GEN, GEN)
    long rnfisfree(GEN, GEN)
    long rnfislocalcyclo(GEN)
    GEN rnfisnorm(GEN, GEN, long)
    GEN rnfisnorminit(GEN, GEN, long)
    GEN rnfkummer(GEN, GEN, long, long)
    GEN rnflllgram(GEN, GEN, GEN, long)
    GEN rnfnormgroup(GEN, GEN)
    GEN rnfpolred(GEN, GEN, long)
    GEN rnfpolredabs(GEN, GEN, long)
    GEN rnfpolredbest(GEN, GEN, long)
    GEN rnfpseudobasis(GEN, GEN)
    GEN rnfsteinitz(GEN, GEN)
    GEN round0(GEN, GEN*)
    GEN select0(GEN, GEN, long)
    GEN pari_self()
    GEN seralgdep(GEN, long, long)
    GEN serchop(GEN, long)
    GEN convol(GEN, GEN)
    GEN laplace(GEN)
    GEN gpserprec(GEN, long)
    GEN serreverse(GEN)
    GEN setbinop(GEN, GEN, GEN)
    GEN setintersect(GEN, GEN)
    long setisset(GEN)
    GEN setminus(GEN, GEN)
    void setrand(GEN)
    long setsearch(GEN, GEN, long)
    GEN setunion(GEN, GEN)
    GEN gshift(GEN, long)
    GEN gmul2n(GEN, long)
    GEN sumdivk(GEN, long)
    int gsigne(GEN)
    GEN simplify(GEN)
    GEN gsin(GEN, long)
    GEN gsinc(GEN, long)
    GEN gsinh(GEN, long)
    long gsizebyte(GEN)
    long sizedigit(GEN)
    GEN gsqr(GEN)
    GEN gsqrt(GEN, long)
    GEN sqrtint(GEN)
    GEN gsqrtn(GEN, GEN, GEN*, long)
    GEN sqrtnint(GEN, long)
    GEN stirling(long, long, long)
    GEN subgrouplist0(GEN, GEN, long)
    GEN gsubst(GEN, long, GEN)
    GEN gsubstpol(GEN, GEN, GEN)
    GEN gsubstvec(GEN, GEN, GEN)
    GEN sumdedekind(GEN, GEN)
    GEN sumdigits0(GEN, GEN)
    GEN sumeulerrat(GEN, GEN, long, long)
    GEN sumformal(GEN, long)
    GEN sumnumapinit(GEN, long)
    GEN sumnuminit(GEN, long)
    GEN sumnumlagrangeinit(GEN, GEN, long)
    GEN sumnummonieninit(GEN, GEN, GEN, long)
    GEN sumnumrat(GEN, GEN, long)
    void gpsystem(char *)
    GEN gtan(GEN, long)
    GEN gtanh(GEN, long)
    GEN tayl(GEN, long, long)
    GEN teichmuller(GEN, GEN)
    GEN theta(GEN, GEN, long)
    GEN thetanullk(GEN, long, long)
    GEN thue(GEN, GEN, GEN)
    GEN thueinit(GEN, long, long)
    GEN gtrace(GEN)
    GEN trunc0(GEN, GEN*)
    GEN type0(GEN)
    GEN gpvaluation(GEN, GEN)
    GEN varhigher(char *, long)
    GEN gpolvar(GEN)
    GEN variables_vec(GEN)
    GEN varlower(char *, long)
    GEN extract0(GEN, GEN, GEN)
    GEN vecmax0(GEN, GEN*)
    GEN vecmin0(GEN, GEN*)
    GEN vecprod(GEN)
    long vecsearch(GEN, GEN, GEN)
    GEN vecsort0(GEN, GEN, long)
    GEN vecsum(GEN)
    GEN pari_version()
    GEN weber0(GEN, long, long)
    void gpwritebin(char *, GEN)
    GEN gzeta(GEN, long)
    GEN zetahurwitz(GEN, GEN, long, long)
    GEN zetamult0(GEN, GEN, long)
    GEN zetamultall(long, long)
    GEN zetamultconvert(GEN, long)
    GEN zetamultinit(long, long)
    GEN znchar(GEN)
    GEN zncharconductor(GEN, GEN)
    GEN znchardecompose(GEN, GEN, GEN)
    GEN znchargauss(GEN, GEN, GEN, long)
    GEN zncharinduce(GEN, GEN, GEN)
    long zncharisodd(GEN, GEN)
    GEN znchartokronecker(GEN, GEN, long)
    GEN znchartoprimitive(GEN, GEN)
    GEN znconreychar(GEN, GEN)
    GEN znconreyconductor(GEN, GEN, GEN*)
    GEN znconreyexp(GEN, GEN)
    GEN znconreylog(GEN, GEN)
    GEN zncoppersmith(GEN, GEN, GEN, GEN)
    GEN znlog0(GEN, GEN, GEN)
    GEN znorder(GEN, GEN)
    GEN znprimroot(GEN)
    GEN znstar0(GEN, long)



cimport cython
from cpython.object cimport PyObject
from .types cimport GEN, pari_sp


cdef class Gen_base:
    # The actual PARI GEN
    cdef GEN g


cdef class Gen(Gen_base):
    # There are 3 kinds of memory management for a GEN:
    # * stack: GEN on the PARI stack
    # * clone: refcounted clone on the PARI heap
    # * constant: universal constant such as gen_0
    #
    # A priori, it makes sense to have separate classes for these cases.
    # However, a GEN may be moved from the stack to the heap. This is
    # easier to support when there is just one class. Second, the
    # differences between the cases are really implementation details
    # which should not affect the user.

    # Base address of the GEN that we wrap. On the stack, this is the
    # value of avma when new_gen() was called. For clones, this is the
    # memory allocated by gclone(). For constants, this is NULL.
    cdef GEN address

    cdef inline pari_sp sp(self) noexcept:
        return <pari_sp>self.address

    # The Gen objects on the PARI stack form a linked list, from the
    # bottom to the top of the stack. This makes sense since we can only
    # deallocate a Gen which is on the bottom of the PARI stack. If this
    # is the last object on the stack, then next = top_of_stack
    # (a singleton object).
    #
    # In the clone and constant cases, this is None.
    cdef Gen next

    # A cache for __getitem__. Initially, this is None but it will be
    # turned into a dict when needed.
    cdef dict itemcache

    cdef inline int cache(self, key, value) except -1:
        """
        Add ``(key, value)`` to ``self.itemcache``.
        """
        if self.itemcache is None:
            self.itemcache = {}
        self.itemcache[key] = value

    cdef Gen new_ref(self, GEN g)

    cdef GEN fixGEN(self) except NULL

    cdef GEN ref_target(self) except NULL


cdef inline Gen Gen_new(GEN g, GEN addr):
    z = <Gen>Gen.__new__(Gen)
    z.g = g
    z.address = addr
    return z


cdef Gen list_of_Gens_to_Gen(list s)
cpdef Gen objtogen(s)
